//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "_DRAW_3D.h"
#include "_3D.h"

//////////////////////////////////////////////////////////////////////////////
//    /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#define PartCount 1024
#define CoordMax 400

tfloat SPEED = 2;    //  
int PHASE = 0;       //  

//////////////////////////////////////////////////////////////////////////////
//  .../////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
tvect(*EForce)(tvect)= NULL;    // 
tfloat (*EPhase)(int)= NULL;  // 
int EConv = 0;

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline void VectLim(tvect &v, tfloat l)
{
  if (v.x > l) v.x = l; else if (v.x < -l) v.x = -l; 
  if (v.y > l) v.y = l; else if (v.y < -l) v.y = -l; 
  if (v.z > l) v.z = l; else if (v.z < -l) v.z = -l; 
};

inline dwRGB DIColor(dwRGB c)
{ 
  return RGBmultV(c,DrawIntensityI);
};

//////////////////////////////////////////////////////////////////////////////
//      ///////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void (*DrawBrush)(int, int, tfloat, tfloat, dwRGB);

void DrawBrush01(int x, int y, tfloat r, tfloat p, dwRGB c)
{ c = DIColor(c); 
  DrawCircle(x, y, r, c, modeADD);
  DrawCircle(x, y, r * 0.5 , c, modeADD);
};

void DrawBrush02(int x, int y, tfloat r, tfloat p, dwRGB c)
{ c = DIColor(c); 
  DrawCircle(x, y, r, c, modeADD);
  DrawCircle(x, y, r * 0.5 , c, modeADD);
  DrawCircle(x + r * 2 * sin(p), y + r * 2 * cos(p), r * 0.25 , c, modeADD);
};

void DrawBrush03(int x, int y, tfloat r, tfloat p, dwRGB c)
{ c = DIColor(c); 
  for (int i = 0; i < 10; i++)
  DrawCircle(x + fRandom(r), y + fRandom(r), fRandom(0.1 * r, 0.3 * r) , c, modeADD);
};

void DrawBrush04(int x, int y, tfloat r, tfloat p, dwRGB c)
{ c = DIColor(c); 
  DrawCircle(x, y, r, c, modeADD);
  DrawCircle(x, y, r * 0.8 , RGBmultV(c, 100), modeSUB);
  DrawCircle(x, y, r * 0.35 , c, modeADD);
  DrawCircle(x, y, r * 0.15 , c, modeADD);
};

void DrawBrush05(int x, int y, tfloat r, tfloat p, dwRGB c)
{ c = DIColor(c); 
  DrawCircle(x, y, r, c, modeADD);
  DrawCircle(x, y, r * 0.8 , RGBmultV(c, 100), modeSUB);
  DrawCircle(x, y, r * 0.35 , c, modeADD);
  DrawCircle(x, y, r * 0.15 , c, modeADD);
  for (int i = 0; i < 12; i++)
  DrawCircle(x + r * 1.75 * sin(p + M_PI / 6 * i), y + r * 1.75 * cos(p + M_PI / 6 * i), r * 0.2 , c, modeADD);
  for (int i = 0; i < 6; i++)
  DrawCircle(x + r * 3 * sin(-p + M_PI / 3 * i), y + r * 3 * cos(-p + M_PI / 3 * i), r * 0.15 , c, modeADD);

};

int SetDrawBrush(int b)
{ switch(b)
  { case 0: 
    case 1: DrawBrush = DrawBrush01; break;
    case 2: DrawBrush = DrawBrush02; break;
    case 3: DrawBrush = DrawBrush03; break;
    case 4: DrawBrush = DrawBrush04; break;
    case 5: DrawBrush = DrawBrush05; break;
  };
  return Yes;
};

int InitDrawBrush = SetDrawBrush(0); 

//////////////////////////////////////////////////////////////////////////////
//  -  //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
class PartBase
{ public:

  tvect P,I;
  tfloat Rad, Speed;
  dwRGB Color;
  int Phase;

  PartBase()
  { P.Rand(CoordMax); I.Rand(2); Rad = 7 + fRandom(7);
    Color = RGBis(50 + Random(35),30 + Random(21), 20 + Random(15));
    Phase = Random(255);
    Speed = 0.5 + fRandom(0.5);
  };

  void Set(tvect p, tfloat r, dwRGB c)  { P = p; Rad = r; Color = c; };

  void Regenerate()
  { if (EForce) I += EForce(P);
    VectLim(I, 7);  
    Phase++; Phase %= 0x0fffffff;
    P += I * SPEED * Speed; I += tvect(fRandom(-1, 1), fRandom(-1, 1), fRandom(-1, 1));
  };

  void Draw()
  { tfloat r, p;
    int x, y;
    dwRGB c;
    DrawPrepare(x, y, r, p, c);
    DrawBrush(x, y, r, p, c);
  };

  void RegenerateAndDraw() { Regenerate(); Draw();  };

  protected:

  void DrawPrepare(int &x, int &y, tfloat &r, tfloat &ph, dwRGB &c)
  { tvect p;
    if (EConv) p = QCNVpoint(P); else p = P;
    tfloat k = DrawPspK(p.z); 
    x = p.x * k * DrawScale750 + DrawXC + 0.5;
    y = p.y * k * DrawScale750 + DrawYC + 0.5;
    if (EPhase) ph = EPhase(Phase); else ph = 1;
    r = k * Rad * DrawScale750;
    c = Color;
  };
};

//////////////////////////////////////////////////////////////////////////////
//   ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
tvect EForce01(tvect p)
{  tvect f = 0;
  if (p.x < -CoordMax) f.x = 1; else if (p.x > CoordMax) f.x = -1;
  if (p.y < -CoordMax) f.y = 1; else if (p.y > CoordMax) f.y = -1;
  if (p.z < -CoordMax) f.z = 1; else if (p.z > CoordMax) f.z = -1;
  return f;
};

tfloat EPhase01(int p) { return 1.1 + sin((tfloat)p / 15); };

tfloat EPhase02(int p) { return (tfloat)p / 15; };


void Falloff(int f)
{ DrawRect(RegionFull(),RGBis(f,f,f),modeMUL);
};

//////////////////////////////////////////////////////////////////////////////
//     /////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
TextSPR *info;

void CommonsInit()
{ RandomSeed();

  FC_FpsSet(AUTO);             //    
  FC_DrawPaint = Yes;          //  fps   DrawPaint();
  
  DrawPspMax = 5;              //    
  DrawSetPspClipPlanes(-115); //    , ,     
                               //   ,       
  //DrawSetFogPlanes(500,-115,effSEXPONENT); //     .

  Draw3D_Init();
  ForceActivate();
  ScreenLoadWait();

  info = new TextSPR(L"TYPE_EN_RU");

  DrawClear();
};

void CommonsDone()
{
  delete info;
  Draw3D_Done();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
int HelpTimer = 0;

void DrawInfo()
{
  tRegion r = RegionCenterIs(200, 120);
  DrawRegionStore.Save(r);
  DrawRect(r, RGBis(50, 50, 50), modeMUL); 
  info->Place(0, 0, modeADD + fromCC, "Control Keys:\n\n\nF1 - hide/show Help\n\nEsc - Exit"); info->Draw();
};

//////////////////////////////////////////////////////////////////////////////////////////////////
void DrawAll()
{
  if (HelpTimer) DrawInfo();
  DrawPaint();
  if (HelpTimer) { DrawRegionStore.Restore();  HelpTimer--; };
};

//////////////////////////////////////////////////////////////////////////////
//   //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void Main()
{
  PartBase part[PartCount];

  CommonsInit();       //   ;  

  SetDrawIntensity(1); //      
  DrawScaleSet(1);     // 

  SetDrawBrush(5);     // 

  OBJ.Current(1);           //   3D 
  COBJ->SetACenter(0,0,0);  // 
  COBJ->SetMap(0,0,0);
  COBJ->SetSMap(0,1,0);
  COBJ->SetSCirc(M_PI/350);

  EForce = EForce01;         //  ""  
  EPhase = EPhase02;
  EConv = 1;

  SPEED = 0.5;               // 

  while (Yes)
   { OBJs_RotateQDec(); QCNVset(1);   
     Falloff(100);
     for (int i = 0; i < PartCount; i++) part[i].RegenerateAndDraw();
     DrawAll();
     if (II.AnyKeyUp)
     { if (II.KeyUp[_Esc_]) break;
       if (II.KeyUp[_F1_]) if (HelpTimer) HelpTimer = 0; else HelpTimer = 25 * 10;
       II.Reset();
     };
   };   

  CommonsDone();       //   ;
};

//////////////////////////////////////////////////////////////////////////////
int LetProcess()
{
  WinName = L"MyDraw";          //    
  VS_BColor = RGBis(25,25,25); //   
  VS_CColor = RGBis(0,0,0);    //   
  DrawSet(FullScreen);         //   ( )  , FullScreen   
  //SetDraw(640,480);
  ProcessFunc = Main;           //    -  
  return on;
};
int Init_LetProcess = LetProcess();

